// C Source File
// Created 9/14/2002; 6:21:59 PM

/*****************************************************
*
*                     Includes
*
*****************************************************/

#include "Snake.h"
#include <stdio.h>
#include <alloc.h>
#include <string.h>
#include <args.h>


/*****************************************************
*
*                 Global variables
*
*****************************************************/

static const char *scorefile = "snkhiscr"; //The name of the hiscore file
const char *filetype = "Snk"; //The file extension


/*****************************************************
*
*                    Functions
*
*****************************************************/


/************************************************
*
*  saveScores() - Save the hiscores
*
*  SCORE *hiscr - The scoreboard to save
*
*  return int  -  1 for success, 0 for failure
*
************************************************/

int saveScores(SCORE *hiscr) {
	FILE *f;
	
	unlink(scorefile);
	
	if((f = fopen(scorefile, "wb")) == NULL)
		return 0;
	if(fwrite(hiscr, sizeof(SCORE), 15, f) != 15)
		return 0;
	
	fputc(0, f);
	fputs(filetype, f);
	fputc(0, f);
	fputc(OTH_TAG, f);
	
	fclose(f);
	
	return 1;
}


/************************************************
*
*  loadScores() - Loads a scoreboard
*
*  SCORE *hiscr - Points to a pre-allocated scoreboard
*
*  return int  -  1 for success, 0 for failure
*
************************************************/

int loadScores(SCORE *hiscr) {
	FILE *f;
	
	if((f = fopen(scorefile, "rb")) == NULL)
		return 0;
	if(fread(hiscr, sizeof(SCORE), 15, f) != 15)
		return 0;
	
	fclose(f);
	
	return 1;
}


/************************************************
*
*  createScoreList() - Creates a new scoreboard in memory
*
*  return int - 1 for success, 0 for failure
*
************************************************/

int createScoreList(void) {
	SCORE board[15];
	
	memset(board,0,sizeof(SCORE) * 15);
	
	return (saveScores(board));
}


/************************************************
*
*  needScoreList() - Checks to see whether a new
*                     score board is neeeded
*
*  return int - 1 for yes, 0 for no
*
************************************************/

int needScoreList(void) {
	FILE *f = fopen(scorefile,"rb");
	
	if (f == NULL)
		return 1;
	
	fclose(f);
	
	return 0;
}


/************************************************
*
*  saveBoardFile() - Saves a board to memory
*
*  char *board - The board to save
*  char *name  - The board/file name to save
*
*  return int  - 1 for success, 0 for failure
*
************************************************/

int saveBoardFile(char *board, char *name) {
	FILE *f;
	int i = 0, j = 0;
	char attr[BOARD_TO_DATA];
	unsigned short numAlloc;
	unsigned long *data;
	unsigned long mask = 0x80000000;
	
	for(i = 0; i < BOARD_TO_DATA; i++) {
		attr[i] = *(board+i);
	}
	
	numAlloc = (BOARDHEIGHT*sizeof(unsigned long));
	data = malloc(numAlloc);
	memset(data, 0, numAlloc);
	
	//Board compression: from big array into a few longs
	for(i = 0; i < BOARDWIDTH; i++) {
		for(j = 0; j < BOARDHEIGHT; j++) {
			if(BOARDPOS(i, j) == BOARD_OBSTACLE)
				*(data+j) |= mask;
		}
		mask >>= 1;
	}
	
	unlink(name);
	
	if((f = fopen(name, "wb")) == NULL) {
		free(data);
		return 0;
	}
	if(fwrite(attr, sizeof(char), BOARD_TO_DATA, f) != BOARD_TO_DATA) {
		free(data);
		return 0;
	}
	if(fwrite(&numAlloc, sizeof(unsigned short), 1, f) != 1) {
		free(data);
		return 0;
	}
	if(fwrite(data, sizeof(unsigned long), numAlloc, f) != numAlloc) {
		free(data);
		return 0;
	}
	
	fputc(0, f);
	fputs("SLvl", f);
	fputc(0, f);
	fputc(OTH_TAG, f);
	
	fclose(f);
	
	free(data);
	
	return 1;
}


/************************************************
*
*  loadBoardFile() - Loads a board from memory
*
*  char *name  -  The board/file name to load
*
*  return char* - The loaded board, NULL on failure
*
************************************************/

char* loadBoardFile(char *name) {
	FILE *f;
	char *board = NULL;
	int i = 0, j = 0;
	char attr[BOARD_TO_DATA];
	unsigned short numAlloc = 0;
	unsigned long *data;
	unsigned long mask = 0x80000000;
	
	if((f = fopen(name, "rb")) == NULL) {
		return NULL;
	}
	if(fread(attr, sizeof(char), BOARD_TO_DATA, f) != BOARD_TO_DATA) {
		return NULL;
	}
	if(fread(&numAlloc, sizeof(unsigned short), 1, f) != 1) {
		return NULL;
	}
	
	data = malloc(numAlloc);
	//memset(data, 0, numAlloc);
	
	if(fread(data, sizeof(unsigned long), numAlloc/4, f) != numAlloc/4) {
		free(data);
		return NULL;
	}
	
	fclose(f);
	
	board = malloc(attr[0]*attr[1]+BOARD_TO_DATA);
	memset(board, 0, attr[0]*attr[1]+BOARD_TO_DATA);
	
	for(i = 0; i < BOARD_TO_DATA; i++) {
		*(board+i) = attr[i];
	}
	
	//Board decompression
	for(i = 0; i < BOARDWIDTH; i++) {
		for(j = 0; j < BOARDHEIGHT; j++) {
			BOARDPOS(i, j) = (*(data+j) & mask) ? BOARD_OBSTACLE : BOARD_EMPTY;
		}
		mask >>= 1;
	}
	
	free(data);
	
	return board;
}
