// C Source File
// Created 9/3/2002; 5:29:43 PM

/*****************************************************
*
*                     Includes
*
*****************************************************/

#include "Snake.h"
#include "Graphx.h"
#include <string.h>
#include <alloc.h>
#include <graph.h>
#include <stdlib.h>
#include <system.h>
#include <peekpoke.h>


/*****************************************************
*
*                    Functions
*
*****************************************************/


/************************************************
*
*  updateSnake() - Updates the given snake structure
*
*  SNAKE *snake - The snake to update
*  char *board  - The board to check against,
*                  NULL if no checking required
*
************************************************/

void updateSnake(SNAKE *snake, char *board) {
	//                 S- S  W  N  E, W- S  W  N  E, N- S  W  N  E, E- S  W  N  E
	char curve[4][4] = {{-1, 0,-1, 1},  {2,-1, 1,-1}, {-1, 3,-1, 2},  {3,-1, 0,-1}};
	int i = 0;
	POS tempTail, tempHead;
	int dx = 4, dy = 1;
	
	if(board) {
		//dx = (33 - BOARDWIDTH)/2;
		//dy = (21 - BOARDHEIGHT)/2;
		dx = (33 - BOARDWIDTH)/2 - BOARDWIDTH%2;
		dy = (21 - BOARDHEIGHT)/2 - BOARDHEIGHT%2;
	}
	
	if(snake->grow) {
		snake->grow--;
		snake->len++;
	}
	else {
		//Assert tail direction
		if(snake->tail.x == snake->turn->pos.x && snake->tail.y == snake->turn->pos.y) {
			snake->tailDir = snake->turn->dir;
			for(i = 0; i < 49; i++) {
				snake->turn[i].pos.x = snake->turn[i+1].pos.x;
				snake->turn[i].pos.y = snake->turn[i+1].pos.y;
				snake->turn[i].dir = snake->turn[i+1].dir;
			}
			snake->turn[49].pos.x = -1;
			snake->turn[49].pos.y = -1;
			snake->turn[49].dir = NOT_A_DIR;
			snake->turns--;
		}
		
		//Erase old tail
		EraseSnake(snake->tail.x, snake->tail.y);
		if(board) {
			tempTail.x = (snake->tail.x)/5 - dx;
			tempTail.y = (snake->tail.y-2)/5 - dy;
			BOARDPOS(tempTail.x, tempTail.y) = BOARD_EMPTY;
		}
		
		//check to make sure tail doesn't overshoot head
		if(snake->len == 1)
			snake->tailDir = snake->headDir;
		
		//Relocate tail
		switch(snake->tailDir) {
		case DIR_SOUTH:
			snake->tail.y += 5;
			break;
		case DIR_WEST:
			snake->tail.x -= 5;
			break;
		case DIR_NORTH:
			snake->tail.y -= 5;
			break;
		case DIR_EAST:
			snake->tail.x += 5;
			break;
		}
		
		//Check for wrapping
		if(board) {
			tempTail.x = (snake->tail.x)/5 - dx;
			tempTail.y = (snake->tail.y-2)/5 - dy;
			if(tempTail.x < 0)
				snake->tail.x += BOARDWIDTH*5;
			if(tempTail.y < 0)
				snake->tail.y += BOARDHEIGHT*5;
			if(tempTail.x >= BOARDWIDTH)
				snake->tail.x -= BOARDWIDTH*5;
			if(tempTail.y >= BOARDHEIGHT)
				snake->tail.y -= BOARDHEIGHT*5;
		}
	}
	
	//Draw tail
	if(snake->tailDir != NOT_A_DIR)
		DrawSnakeTail(snake->tail.x, snake->tail.y, snake->tailDir);
	
	//move body section over head section
	if(snake->len > 2) {
		if(snake->headDir == snake->lastDir)
			DrawSnakeBody(snake->head.x, snake->head.y, snake->headDir);
		else if (snake->headDir == NOT_A_DIR) {
			DrawSnakeBody(snake->head.x, snake->head.y, snake->lastDir);
		}
		else
			DrawSnakeCurve(snake->head.x, snake->head.y, curve[(int)(snake->lastDir)][(int)(snake->headDir)]);
	}
	
	//Relocate head
	snake->lastDir = snake->headDir;
	switch(snake->headDir) {
	case DIR_SOUTH:
		snake->head.y += 5;
		break;
	case DIR_WEST:
		snake->head.x -= 5;
		break;
	case DIR_NORTH:
		snake->head.y -= 5;
		break;
	case DIR_EAST:
		snake->head.x += 5;
		break;
	case NOT_A_DIR:
		snake->len--;
		break;
	}
	
	//Check for death of snake
	if(board && snake->headDir != NOT_A_DIR) {
		tempHead.x = (snake->head.x)/5 - dx;
		tempHead.y = (snake->head.y-2)/5 - dy;
		if(tempHead.x < 0 || tempHead.y < 0 || tempHead.x >= BOARDWIDTH || tempHead.y >= BOARDHEIGHT) {
			if(!BOARDWRAP) {
				snake->dead = DEATH_OUT_OF_BOUNDS;
				snake->lives--;
				return;
			}
			else {
				if(tempHead.x < 0) {
					snake->head.x += BOARDWIDTH*5;
					tempHead.x = BOARDWIDTH-1;
				}
				if(tempHead.y < 0) {
					snake->head.y += BOARDHEIGHT*5;
					tempHead.y = BOARDHEIGHT-1;
				}
				if(tempHead.x >= BOARDWIDTH) {
					snake->head.x -= BOARDWIDTH*5;
					tempHead.x = 0;
				}
				if(tempHead.y >= BOARDHEIGHT) {
					snake->head.y -= BOARDHEIGHT*5;
					tempHead.y = 0;
				}
			}
		}
		if(BOARDPOS(tempHead.x, tempHead.y) > BOARD_APPLE) {
			snake->dead = BOARDPOS(tempHead.x, tempHead.y);
			snake->lives--;
			return;
		}
		if(BOARDPOS(tempHead.x, tempHead.y) == BOARD_APPLE) snake->eating++;
		BOARDPOS(tempHead.x, tempHead.y) = BOARD_SNAKE;
	}
	
	//Redraw head
	if(snake->len && snake->headDir != NOT_A_DIR)
		DrawSnakeHead(snake->head.x, snake->head.y, snake->headDir);
	
}


/************************************************
*
*  doIntro() - Displays the Snake title
*
************************************************/

void doIntro(void) {
	SNAKE snake[6];
	int i = 0;
	
	for(i = 0; i < 6; i++)
		initSnake(snake + i);
	
	//ClearGrayScreen();
	wipeScreen();
	
	//S
	snake[0].headDir = DIR_EAST;
	snake[0].tailDir = DIR_EAST;
	snake[0].head.x = 20;
	snake[0].head.y = 50;
	snake[0].tail.x = 20;
	snake[0].tail.y = 50;
	snake[0].grow = 12;
	
	for(i = 0; i < 4; i++) {
		optDelay(20);
		updateSnake(snake, NULL);
	}
	snake[0].headDir = DIR_NORTH;
	snake[0].turn[0].pos.x = snake[0].head.x;
	snake[0].turn[0].pos.y = snake[0].head.y;
	snake[0].turn[0].dir = snake[0].headDir;
	for(i = 0; i < 2; i++) {
		optDelay(20);
		updateSnake(snake, NULL);
	}
	snake[0].headDir = DIR_WEST;
	snake[0].turn[1].pos.x = snake[0].head.x;
	snake[0].turn[1].pos.y = snake[0].head.y;
	snake[0].turn[1].dir = snake[0].headDir;
	for(i = 0; i < 2; i++) {
		optDelay(20);
		updateSnake(snake, NULL);
	}
	snake[0].headDir = DIR_NORTH;
	snake[0].turn[2].pos.x = snake[0].head.x;
	snake[0].turn[2].pos.y = snake[0].head.y;
	snake[0].turn[2].dir = snake[0].headDir;
	for(i = 0; i < 2; i++) {
		optDelay(20);
		updateSnake(snake, NULL);
	}
	snake[0].headDir = DIR_EAST;
	snake[0].turn[3].pos.x = snake[0].head.x;
	snake[0].turn[3].pos.y = snake[0].head.y;
	snake[0].turn[3].dir = snake[0].headDir;
	for(i = 0; i < 3; i++) {
		optDelay(20);
		updateSnake(snake, NULL);
	}
	
	//n
	snake[1].headDir = DIR_NORTH;
	snake[1].tailDir = DIR_NORTH;
	snake[1].head.x = 55;
	snake[1].head.y = 55;
	snake[1].tail.x = 55;
	snake[1].tail.y = 55;
	snake[1].grow = 8;
	
	for(i = 0; i < 4; i++) {
		optDelay(20);
		updateSnake(snake+1, NULL);
	}
	snake[1].headDir = DIR_EAST;
	snake[1].turn[0].pos.x = snake[1].head.x;
	snake[1].turn[0].pos.y = snake[1].head.y;
	snake[1].turn[0].dir = snake[1].headDir;
	for(i = 0; i < 2; i++) {
		optDelay(20);
		updateSnake(snake+1, NULL);
	}
	snake[1].headDir = DIR_SOUTH;
	snake[1].turn[1].pos.x = snake[1].head.x;
	snake[1].turn[1].pos.y = snake[1].head.y;
	snake[1].turn[1].dir = snake[1].headDir;
	for(i = 0; i < 3; i++) {
		optDelay(20);
		updateSnake(snake+1, NULL);
	}
	
	//a
	snake[2].headDir = DIR_WEST;
	snake[2].tailDir = DIR_WEST;
	snake[2].head.x = 80;
	snake[2].head.y = 40;
	snake[2].tail.x = 80;
	snake[2].tail.y = 40;
	snake[2].grow = 10;
	
	//for(i = 0; i < 1; i++) {
		optDelay(20);
		updateSnake(snake+2, NULL);
	//}
	snake[2].headDir = DIR_SOUTH;
	snake[2].turn[0].pos.x = snake[2].head.x;
	snake[2].turn[0].pos.y = snake[2].head.y;
	snake[2].turn[0].dir = snake[2].headDir;
	for(i = 0; i < 2; i++) {
		optDelay(20);
		updateSnake(snake+2, NULL);
	}
	snake[2].headDir = DIR_EAST;
	snake[2].turn[1].pos.x = snake[2].head.x;
	snake[2].turn[1].pos.y = snake[2].head.y;
	snake[2].turn[1].dir = snake[2].headDir;
	for(i = 0; i < 2; i++) {
		optDelay(20);
		updateSnake(snake+2, NULL);
	}
	snake[2].headDir = DIR_NORTH;
	snake[2].turn[2].pos.x = snake[2].head.x;
	snake[2].turn[2].pos.y = snake[2].head.y;
	snake[2].turn[2].dir = snake[2].headDir;
	for(i = 0; i < 4; i++) {
		optDelay(20);
		updateSnake(snake+2, NULL);
	}
	snake[2].headDir = DIR_WEST;
	snake[2].turn[3].pos.x = snake[2].head.x;
	snake[2].turn[3].pos.y = snake[2].head.y;
	snake[2].turn[3].dir = snake[2].headDir;
	//for(i = 0; i < 1; i++) {
		optDelay(20);
		updateSnake(snake+2, NULL);
	//}
	
	//k
	snake[3].headDir = DIR_SOUTH;
	snake[3].tailDir = DIR_SOUTH;
	snake[3].head.x = 95;
	snake[3].head.y = 20;
	snake[3].tail.x = 95;
	snake[3].tail.y = 20;
	snake[3].grow = 5;
	
	for(i = 0; i < 6; i++) {
		optDelay(20);
		updateSnake(snake+3, NULL);
	}
	
	snake[4].headDir = DIR_NORTH;
	snake[4].tailDir = DIR_NORTH;
	snake[4].head.x = 105;
	snake[4].head.y = 55;
	snake[4].tail.x = 105;
	snake[4].tail.y = 55;
	snake[4].grow = 5;
	
	for(i = 0; i < 2; i++) {
		optDelay(20);
		updateSnake(snake+4, NULL);
	}
	snake[4].headDir = DIR_WEST;
	snake[4].turn[0].pos.x = snake[4].head.x;
	snake[4].turn[0].pos.y = snake[4].head.y;
	snake[4].turn[0].dir = snake[4].headDir;
	//for(i = 0; i < 1; i++) {
		optDelay(20);
		updateSnake(snake+4, NULL);
	//}
	snake[4].headDir = DIR_NORTH;
	snake[4].turn[1].pos.x = snake[4].head.x;
	snake[4].turn[1].pos.y = snake[4].head.y;
	snake[4].turn[1].dir = snake[4].headDir;
	//for(i = 0; i < 1; i++) {
		optDelay(20);
		updateSnake(snake+4, NULL);
	//}
	snake[4].headDir = DIR_EAST;
	snake[4].turn[2].pos.x = snake[4].head.x;
	snake[4].turn[2].pos.y = snake[4].head.y;
	snake[4].turn[2].dir = snake[4].headDir;
	//for(i = 0; i < 1; i++) {
		optDelay(20);
		updateSnake(snake+4, NULL);
	//}
	snake[4].headDir = DIR_NORTH;
	snake[4].turn[3].pos.x = snake[4].head.x;
	snake[4].turn[3].pos.y = snake[4].head.y;
	snake[4].turn[3].dir = snake[4].headDir;
	//for(i = 0; i < 1; i++) {
		optDelay(20);
		updateSnake(snake+4, NULL);
	//}
	
	//e
	snake[5].headDir = DIR_EAST;
	snake[5].tailDir = DIR_EAST;
	snake[5].head.x = 120;
	snake[5].head.y = 45;
	snake[5].tail.x = 120;
	snake[5].tail.y = 45;
	snake[5].grow = 9;
	
	//for(i = 0; i < 1; i++) {
		optDelay(20);
		updateSnake(snake+5, NULL);
	//}
	snake[5].headDir = DIR_NORTH;
	snake[5].turn[0].pos.x = snake[5].head.x;
	snake[5].turn[0].pos.y = snake[5].head.y;
	snake[5].turn[0].dir = snake[5].headDir;
	//for(i = 0; i < 1; i++) {
		optDelay(20);
		updateSnake(snake+5, NULL);
	//}
	snake[5].headDir = DIR_WEST;
	snake[5].turn[1].pos.x = snake[5].head.x;
	snake[5].turn[1].pos.y = snake[5].head.y;
	snake[5].turn[1].dir = snake[5].headDir;
	for(i = 0; i < 2; i++) {
		optDelay(20);
		updateSnake(snake+5, NULL);
	}
	snake[5].headDir = DIR_SOUTH;
	snake[5].turn[2].pos.x = snake[5].head.x;
	snake[5].turn[2].pos.y = snake[5].head.y;
	snake[5].turn[2].dir = snake[5].headDir;
	for(i = 0; i < 2; i++) {
		optDelay(20);
		updateSnake(snake+5, NULL);
	}
	snake[5].headDir = DIR_EAST;
	snake[5].turn[3].pos.x = snake[5].head.x;
	snake[5].turn[3].pos.y = snake[5].head.y;
	snake[5].turn[3].dir = snake[5].headDir;
	for(i = 0; i < 3; i++) {
		optDelay(20);
		updateSnake(snake+5, NULL);
	}
	
	optDelay(DELAY_SECOND * 4);
	
	for(i = 0; i < 6; i++) snake[i].headDir = NOT_A_DIR;
	for(i = 0; i < 13; i++) {
		updateSnake(snake+0, NULL);
		updateSnake(snake+1, NULL);
		updateSnake(snake+2, NULL);
		updateSnake(snake+3, NULL);
		updateSnake(snake+4, NULL);
		updateSnake(snake+5, NULL);
		optDelay(20);
	}
	
	for(i = 0; i < 6; i++) free(snake[i].turn);
	
}


/************************************************
*
*  drawHUD() - Draws the Heads-Up Display for the game
*
*  SNAKE *snake - A pointer to the snake
*
************************************************/

void drawHUD(SNAKE *snake) {
	int i = 0;
	
	for(i = 0; i < snake->lives; i++) {
		//DrawLive(5+i*7, 2);
		DrawSnakeHead(5+i*7, 2, DIR_NORTH);
	}
	
	DrawApple(120, 2);
	drawChar(130, 1, pickChar('='), TEXT_BLACK | TEXT_OR);
}


/************************************************
*
*  updateHUD() - Updates the score and apple displays
*
*  int appleNum - The number of apples remaining
*  int score   -  The score
*
************************************************/

void updateHUD(int appleNum, unsigned long score) {
	char str[6];
	int len = 1;
	
	sprintf(str, "%lu", score);
	len = (int)strlen(str);
	
	DrawGrayRect(80-len*4, 0, 80+len*4, 7, COLOR_WHITE, RECT_FILLED);
	drawText(80-len*4, 0, str, TEXT_OR | TEXT_BLACK);
	
	/*if(appleNum < 0) {
		DrawInfinity(139, 0);
		return;
	}*/
	
	if(appleNum >= 100) {
		EraseFont(130, 0);
		DrawNum(130, 0, appleNum / 100);
		appleNum %= 100;
	}
	
	EraseFont(138, 0);
	if(appleNum >= 10)
		DrawNum(138, 0, appleNum / 10);
	
	EraseFont(146, 0);
	DrawNum(146, 0, appleNum % 10);
}


/************************************************
*
*  drawBoard() - Draws the board
*
*  char *board - The board to draw
*  int shift  -  Whether to shift the board down
*
************************************************/

void drawBoard(char *board, int shift) {
	POS upLeft, downRight;
	int i = 0, j = 0;
	
	//upLeft.x = ((33 - BOARDWIDTH) / 2) * 5 - 1;
	//upLeft.y = ((21 - BOARDHEIGHT) / 2) * 5 - 1;
	upLeft.x = 80 - (BOARDWIDTH*5/2) - 1;
	upLeft.y = 50 - (BOARDHEIGHT*5/2) - 1;
	downRight.x = 159 - upLeft.x;
	downRight.y = 99 - upLeft.y;
	if(BOARDWIDTH%2) downRight.x++;
	if(BOARDHEIGHT%2) downRight.y++;
	if(shift) {
		upLeft.y += 2;
		downRight.y += 2;
	}
	
	ClearGrayScreen();
	/*int color1, color2;
	if(BOARDWRAP) {
		color1 = COLOR_LIGHTGRAY;
		color2 = COLOR_LIGHTGRAY;
	}
	else {
		color1 = COLOR_DARKGRAY;
		color2 = COLOR_BLACK;
	}
	FastDrawGrayLine(upLeft.x, upLeft.y, downRight.x, upLeft.y, color2);
	FastDrawGrayLine(downRight.x, downRight.y, upLeft.x, downRight.y, color1);
	FastDrawGrayLine(upLeft.x, upLeft.y, upLeft.x, downRight.y, color2);
	FastDrawGrayLine(downRight.x, upLeft.y, downRight.x, downRight.y, color1);*/
	if(BOARDWRAP) {
		FastDrawGrayLine(upLeft.x, upLeft.y, downRight.x, upLeft.y, COLOR_LIGHTGRAY);
		FastDrawGrayLine(downRight.x, downRight.y, upLeft.x, downRight.y, COLOR_LIGHTGRAY);
		FastDrawGrayLine(upLeft.x, upLeft.y, upLeft.x, downRight.y, COLOR_LIGHTGRAY);
		FastDrawGrayLine(downRight.x, upLeft.y, downRight.x, downRight.y, COLOR_LIGHTGRAY);
	}
	else {
		for(i = 0; i < BOARDHEIGHT; i++) {
			DrawMultiObst(upLeft.x-4, upLeft.y+i*5+1, (BOARDPOS(0, i) == BOARD_OBSTACLE) ? 0b1011 : 0b1010);
			DrawMultiObst(downRight.x, upLeft.y+i*5+1, (BOARDPOS(BOARDWIDTH-1, i) == BOARD_OBSTACLE) ? 0b1110 : 0b1010);
		}
		for(i = 0; i < BOARDWIDTH; i++) {
			DrawMultiObst(upLeft.x+i*5+1, downRight.y, (BOARDPOS(i, BOARDHEIGHT-1) == BOARD_OBSTACLE) ? 0b0111 : 0b0101);
			DrawMultiObst(upLeft.x+i*5+1, upLeft.y-4, (BOARDPOS(i, 0) == BOARD_OBSTACLE) ? 0b1101 : 0b0101);
		}
		DrawMultiObst(upLeft.x-4, upLeft.y-4, 0b1001);
		DrawMultiObst(upLeft.x-4, downRight.y, 0b0011);
		DrawMultiObst(downRight.x, downRight.y, 0b0110);
		DrawMultiObst(downRight.x, upLeft.y-4, 0b1100);
	}
	
	for(i = 0; i < BOARDHEIGHT; i++)
		for(j = 0; j < BOARDWIDTH; j++)
			if(BOARDPOS(j, i) == BOARD_OBSTACLE)
				DrawMultiObst(j*5 + upLeft.x+1, i*5 + upLeft.y+1, getObstType(board, j, i));
}


/************************************************
*
*  doLevelIntro() - Displays a introduction to a level
*
*  int levelNum - The number of the level
*
************************************************/

void doLevelIntro(int levelNum) {
	int first = 0, second = 0;
	
	if(levelNum > 9) {
		first = levelNum/10;
		second = levelNum%10;
	}
	else
		first = levelNum;
	
	ClearGrayScreen();
	
	Sprite32_OR(60, 40, 9, Level+9, GetPlane(LIGHT_PLANE));
	Sprite8_OR(74, 50, 8, Font+(first*8), GetPlane(LIGHT_PLANE));
	if(levelNum > 9) Sprite8_OR(82, 50, 8, Font+second*8, GetPlane(LIGHT_PLANE));
	
	delay(100);
	
	/*Sprite32_XOR(60, 40, 9, Level+9, GetPlane(LIGHT_PLANE));
	Sprite8_XOR(74, 50, 8, Font+(first*16)+8, GetPlane(LIGHT_PLANE));
	if(levelNum > 9) Sprite8_XOR(82, 50, 8, Font+(second*16)+8, GetPlane(LIGHT_PLANE));*/
	ClearGrayScreen();
	
	Sprite32_OR(60, 40, 9, Level, GetPlane(DARK_PLANE));
	Sprite8_OR(74, 50, 8, Font+first*8, GetPlane(DARK_PLANE));
	if(levelNum > 9) Sprite8_OR(82, 50, 8, Font+second*8, GetPlane(DARK_PLANE));
	
	delay(100);
	
	Sprite32_OR(60, 40, 9, Level+9, GetPlane(LIGHT_PLANE));
	Sprite8_OR(74, 50, 8, Font+first*8, GetPlane(LIGHT_PLANE));
	if(levelNum > 9) Sprite8_OR(82, 50, 8, Font+second*8, GetPlane(LIGHT_PLANE));
	
	while(!getKey());
	
	/*Sprite32_XOR(60, 40, 9, Level+9, GetPlane(LIGHT_PLANE));
	Sprite8_XOR(74, 50, 8, Font+(first*16)+8, GetPlane(LIGHT_PLANE));
	if(levelNum > 9) Sprite8_XOR(82, 50, 8, Font+(second*16)+8, GetPlane(LIGHT_PLANE));
	Sprite32_XOR(90, 40, 9, Level, GetPlane(DARK_PLANE));
	Sprite8_XOR(74, 50, 8, Font+(first*16), GetPlane(DARK_PLANE));
	if(levelNum > 9) Sprite8_XOR(82, 50, 8, Font+(second*16), GetPlane(DARK_PLANE));*/
	ClearGrayScreen();
	
}


/************************************************
*
*  doCustomIntro() - Displays an introduction to
*                     custom-made levels (or freeplay)
*
*  char *name - The name of the level
*
************************************************/

void doCustomIntro(char *name) {
	int i = 0, j = 0, len = (int)strlen(name), x = 80 - len*5;
	
	ClearGrayScreen();
	OSContrastUp();
	FontSetSys(F_6x8);
	//SetPlane(DARK_PLANE);
	for(i = 0; i < len; i++) {
		for(j = 1; j < 46; j++) {
			//DrawChar(x+i*8, j, *(name+i), A_REPLACE);
			//DrawChar(x+i*8, 91-j, *(name+i), A_REPLACE);
			EraseFont(x+i*10, j-1);
			EraseFont(x+i*10, 91-j);
			drawChar(x+i*10, j, pickChar(*(name+i)), TEXT_OR | TEXT_DARK);
			drawChar(x+i*10, 90-j, pickChar(*(name+i)), TEXT_OR | TEXT_DARK);
			//ClearGrayScreen();
		}
		drawChar(x+i*10-1, 46, pickChar(*(name+i)), TEXT_OR | TEXT_LIGHT);
	}
	
	/*for(i = 0; i < len; i++)
		//DrawChar(x+i*8, 46, *(name+i), A_REPLACE);
		drawChar(x+i*10, 46, pickChar(*(name+i)), TEXT_OR | TEXT_DARK);*/
	optDelay(DELAY_SECOND * 5);
	
	for(i = len-1; i >= 0; i--) {
		EraseFont(x+i*10-1, 46);
		for(j = 45; j >= 0; j--) {
			//DrawChar(x+i*8, j+1, ' ', A_REPLACE);
			//DrawChar(x+i*8, 90-j, ' ', A_REPLACE);
			//DrawChar(x+i*8, j, *(name+i), A_REPLACE);
			//DrawChar(x+i*8, 91-j, *(name+i), A_REPLACE);
			EraseFont(x+i*10, j+1);
			EraseFont(x+i*10, 90-j);
			drawChar(x+i*10, j, pickChar(*(name+i)), TEXT_OR | TEXT_DARK);
			drawChar(x+i*10, 91-j, pickChar(*(name+i)), TEXT_OR | TEXT_DARK);
		}
		//DrawChar(x+i*8, 0, ' ', A_REPLACE);
		//DrawChar(x+i*8, 91, ' ', A_REPLACE);
		EraseFont(x+i*10, 0);
		EraseFont(x+i*10, 91);
	}
	OSContrastDn();
}


/************************************************
*
*  doIntro() - Displays a 3-2-1 countdown
*
************************************************/

void countdown(void) {
	int i = 0;
	unsigned char *buff;
	buff = (unsigned char *)malloc(16*sizeof(char));
	Sprite8Get(76, 48, 8, GetPlane(DARK_PLANE), buff);
	Sprite8Get(76, 48, 8, GetPlane(LIGHT_PLANE), buff+8);
	
	for(i = 11; i >= 0; i--) {
		if(i & 1)
			Sprite8_AND(76, 48, 8, FontMask+((i/4)+1)*8, GetPlane(LIGHT_PLANE));
		if(i & 2)
			Sprite8_AND(76, 48, 8, FontMask+((i/4)+1)*8, GetPlane(DARK_PLANE));
		drawChar(76, 48, (i/4)+1, TEXT_OR | (i & 3));
		if(contOptDelay(DELAY_SECOND/4)) {
			do {
				if(getKey() & KEY_ESC) break;
			} while(contOptDelay(0));
		}
		//EraseFont(76, 46);
		GraySprite8_BLIT(76, 48, 8, buff, buff+8, 0, GetPlane(DARK_PLANE), GetPlane(LIGHT_PLANE));
	}
	free(buff);
}


/************************************************
*
*  createApple() - Makes a new apple
*
*  char *board - The board to check against and mark on
*
************************************************/

void createApple(char *board) {
	POS apple;
	
	//randomize();
	
	do {
		apple.x = rand() % (BOARDWIDTH-1);
		apple.y = rand() % (BOARDHEIGHT-1);
	} while(BOARDPOS(apple.x, apple.y) != 0);
	
	BOARDPOS(apple.x, apple.y) = BOARD_APPLE;
	apple.x = (apple.x + (33 - BOARDWIDTH)/2) * 5 - BOARDWIDTH%2 * 2;
	apple.y = (apple.y + (21 - BOARDHEIGHT)/2) * 5 - BOARDHEIGHT%2 * 2+2;
	DrawApple(apple.x, apple.y);
}


/************************************************
*
*  drawDeath() - Displays text for a death
*
*  char type - The type of death:
*                DEATH_OUT_OF_BOUNDS
*                DEATH_OBSTACLE
*                DEATH_SNAKE
*
************************************************/

void drawDeath(char type) {
	int x = 0;
	char str[20];
	
	switch(type) {
	case STATUS_ALIVE:
	case STATUS_NEXT_LEVEL:
	case DEATH_ESC:
		return;
	case DEATH_OUT_OF_BOUNDS:
		sprintf(str, "Out of bounds!");
		x = 24;
		break;
	case DEATH_OBSTACLE:
		sprintf(str, "Hit the wall!");
		x = 28;
		break;
	case DEATH_SNAKE:
		sprintf(str, "Hit yourself!");
		x = 28;
		break;
	}
	
	drawText(x, 44, str, TEXT_BLACK | TEXT_MASK);
	delay(100);
	while(!getKey());
}


/************************************************
*
*  winSequence() - Displays a level-winning sequence
*
*  SNAKE *snake - The snake structure to use
*  char *board  - The board to use for boundary checking
*  int speed   -  How fast to go (lower value = faster)
*
************************************************/

void winSequence(SNAKE *snake, char *board, int speed) {
	int i = 0;
	
	snake->headDir = NOT_A_DIR;
	
	while(snake->len) {
		updateSnake(snake, board);
		delay(speed);
	}
	
	for(i = 0; i < 100; i++) {
		ScrollDown160(GetPlane(DARK_PLANE), 100);
		ScrollDown160(GetPlane(LIGHT_PLANE), 100);
		delay(3);
		optDelay(3);
	}
	
	Sprite32_OR(49, 40, 9, Level+9, GetPlane(LIGHT_PLANE));
	SpriteX8_OR(62, 50, 11, Complete+77, 7, GetPlane(LIGHT_PLANE));
	
	delay(100);
	
	Sprite32_XOR(49, 40, 9, Level+9, GetPlane(LIGHT_PLANE));
	SpriteX8_XOR(62, 50, 11, Complete+77, 7, GetPlane(LIGHT_PLANE));
	
	Sprite32_OR(49, 40, 9, Level, GetPlane(DARK_PLANE));
	SpriteX8_OR(62, 50, 11, Complete, 7, GetPlane(DARK_PLANE));
	
	delay(100);
	
	Sprite32_OR(49, 40, 9, Level+9, GetPlane(LIGHT_PLANE));
	SpriteX8_OR(62, 50, 11, Complete+77, 7, GetPlane(LIGHT_PLANE));
	
	optDelay(DELAY_SECOND * 5);
	
	Sprite32_XOR(49, 40, 9, Level+9, GetPlane(LIGHT_PLANE));
	SpriteX8_XOR(62, 50, 11, Complete+77, 7, GetPlane(LIGHT_PLANE));
	Sprite32_XOR(49, 40, 9, Level, GetPlane(DARK_PLANE));
	SpriteX8_XOR(62, 50, 11, Complete, 7, GetPlane(DARK_PLANE));
	
}


/************************************************
*
*  menu() - Displays the main menu
*
************************************************/

void menu(void) {
	static int sel = MENU_FREEPLAY, subSel = SUBMENU_NONE, speed = SPEED_MEDIUM;
	int key = KEY_NONE, xOffset = 0, subOffset = 0, focus = FOCUS_TOP_MENU, subAvail = 0;
	char str[12] = "Levels", subStr[12] = "Start";
	
	delay(100);
	while(key != KEY_ENTER) {
		switch(sel) {
		case MENU_FREEPLAY:
			sprintf(str, "Freeplay");
			xOffset = 42;
			subAvail = 0;
			break;
		case MENU_LEVELS:
			sprintf(str, "Levels");
			xOffset = 34;
			subAvail = 2;
			switch(subSel) {
			case SUBMENU_LEVELSTART:
				sprintf(subStr, "Start");
				subOffset = 30;
				break;
			case SUBMENU_LEVELWARP:
				sprintf(subStr, "Warp");
				subOffset = 26;
				break;
			}
			break;
		case MENU_LEVELEDITOR:
			sprintf(str, "Editor");
			xOffset = 34;
			subAvail = 4;
			switch(subSel) {
			case SUBMENU_LEVEDITNEW:
				sprintf(subStr, "New");
				subOffset = 22;
				break;
			case SUBMENU_LEVEDITLOAD:
				sprintf(subStr, "Load");
				subOffset = 26;
				break;
			case SUBMENU_LEVEDITPLAY:
				sprintf(subStr, "Play Saved");
				subOffset = 50;
				break;
			case SUBMENU_LEVEDITRAND:
				sprintf(subStr, "Play Random");
				subOffset = 54;
				break;
			}
			break;
		case MENU_OPTIONS:
			sprintf(str, "Options");
			xOffset = 38;
			subAvail = 0;
			break;
		case MENU_ABOUT:
			sprintf(str, "About");
			xOffset = 30;
			subAvail = 0;
			break;
		case MENU_QUIT:
			sprintf(str, "Quit");
			xOffset = 26;
			subAvail = 0;
			break;
		}
		
		ClearGrayScreen();
		if(focus == FOCUS_TOP_MENU)
			//DrawGrayStr(90-xOffset, 45, str, A_REPLACE);
			drawText(90-xOffset, 46, str, TEXT_BLACK | TEXT_OR | TEXT_INTLIGHT);
		else {
			//DrawGrayStr(90-xOffset, 36, str, A_REPLACE);
			//DrawGrayStr(90-subOffset, 54, subStr, A_REPLACE);
			drawText(90-xOffset, 37, str, TEXT_BLACK | TEXT_OR | TEXT_INTDARK);
			drawText(90-subOffset, 55, subStr, TEXT_BLACK | TEXT_OR | TEXT_INTLIGHT);
		}
		
		key = KEY_NONE;
		delay(50);
		while(!(key & (KEY_ENTER | KEY_LEFT | KEY_RIGHT | KEY_DOWN | KEY_UP | KEY_ESC | KEY_BOSS))) {
			optDelay(25);
			key = getKey();
			if(focus == FOCUS_TOP_MENU) {
				XorArrow(78-xOffset, 47, DIR_WEST);
				XorArrow(78+xOffset, 47, DIR_EAST);
			}
			else {
				XorArrow(78-subOffset, 56, DIR_WEST);
				XorArrow(78+subOffset, 56, DIR_EAST);
			}
			if(focus == FOCUS_TOP_MENU && subAvail)
				XorArrow(78, 56, DIR_SOUTH);
			else if(focus == FOCUS_SUB_MENU)
				XorArrow(78, 48, DIR_NORTH);
		}
		if(key & KEY_BOSS)
			return;
		if(key & KEY_LEFT) {
			if(focus == FOCUS_TOP_MENU)
				sel = (sel == MENU_FREEPLAY) ? MENU_QUIT : sel - 1;
			else
				subSel = (subSel == 1) ? subAvail : subSel - 1;
		}
		if(key & KEY_RIGHT) {
			if(focus == FOCUS_TOP_MENU)
				sel = (sel == MENU_QUIT) ? MENU_FREEPLAY : sel + 1;
			else
				subSel = (subSel == subAvail) ? 1 : subSel + 1;
		}
		if(key & KEY_DOWN && focus == FOCUS_TOP_MENU && subAvail) {
			subSel = 1;
			focus = FOCUS_SUB_MENU;
		}
		if(key & KEY_UP && focus == FOCUS_SUB_MENU) {
			subSel = SUBMENU_NONE;
			focus = FOCUS_TOP_MENU;
		}
		if(key & KEY_ESC) {
			if(sel != MENU_QUIT) {
				subSel = SUBMENU_NONE;
				focus = FOCUS_TOP_MENU;
				sel = MENU_QUIT;
			}
			else
				return;
		}
	}
	
	switch(sel) {
	case MENU_FREEPLAY:
		if(checkHiscores(game(speed, GAME_FREEPLAY), GAME_FREEPLAY))
			drawHiscores(1);
		menu();
		break;
	case MENU_LEVELS:
		switch(subSel) {
		case SUBMENU_LEVELSTART:
			if(checkHiscores(game(speed, GAME_LEVELS), GAME_LEVELS))
				drawHiscores(0);
			break;
		case SUBMENU_LEVELWARP:
			if(checkHiscores(game(speed, GAME_WARP), GAME_WARP))
				drawHiscores(0);
			break;
		}
		menu();
		break;
	case MENU_LEVELEDITOR:
		switch(subSel) {
		case SUBMENU_LEVEDITNEW:
			editor(EDITOR_NEW);
			break;
		case SUBMENU_LEVEDITLOAD:
			editor(EDITOR_LOAD);
			break;
		case SUBMENU_LEVEDITPLAY:
			if(checkHiscores(game(speed, GAME_CUSTOM), GAME_CUSTOM))
				drawHiscores(2);
			break;
		case SUBMENU_LEVEDITRAND:
			if(checkHiscores(game(speed, GAME_RANDOM), GAME_RANDOM))
				drawHiscores(2);
			break;
		}
		menu();
		break;
	case MENU_ABOUT:
		about();
		menu();
		break;
	case MENU_OPTIONS:
		options(&speed);
		menu();
		break;
	}
	delay(50);
}

/************************************************
*
*  options() - Displays the game options menu
*
*  int *speed - The variable used for speed
*
************************************************/

void options(int *speed) {
	int i = 0, j = 0, sel = 0;
	int key = KEY_NONE;
	
	openWindow(110, 30);
	
	drawText(35, 39, "Speed", TEXT_BLACK | TEXT_OR);
	drawText(40, 53, "View Scores", TEXT_BLACK | TEXT_OR);
	
	while(!(key & (KEY_ENTER | KEY_ESC))) {
		DrawGrayRect(89, 37, 126, 47, COLOR_WHITE, RECT_FILLED);
		FastDrawGrayHLine(90, 125, 47, COLOR_BLACK);
		for(i = 0; i <= *speed; i++) {
			for(j = 0; j <= i; j++) {
				if(i < 3) {
					FastDrawGrayLine(90+i*4, 47-(j), 93+i*4, 47-(j), COLOR_LIGHTGRAY);
					FastDrawGrayLine(92+i*4, 46-(j), 93+i*4, 46-(j), COLOR_LIGHTGRAY);
				}
				else if(i < 6) {
					FastDrawGrayLine(90+i*4, 47-(j), 93+i*4, 47-(j), COLOR_DARKGRAY);
					FastDrawGrayLine(92+i*4, 46-(j), 93+i*4, 46-(j), COLOR_DARKGRAY);
				}
				else {
					FastDrawGrayLine(90+i*4, 47-(j), 93+i*4, 47-(j), COLOR_BLACK);
					FastDrawGrayLine(92+i*4, 46-(j), 93+i*4, 46-(j), COLOR_BLACK);
				}
			}
		}
		key = KEY_NONE;
		
		delay(50);
		while(!(key & (KEY_ENTER | KEY_ESC | KEY_LEFT | KEY_RIGHT | KEY_UP | KEY_DOWN))) {
			key = getKey();
			if(!sel) XorArrow(28, 40, DIR_EAST);
			else XorArrow(33, 54, DIR_EAST);
			optDelay(15);
		}
		if(!sel) EraseArrow(28, 40);
		else EraseArrow(33, 54);
		
		if(key & KEY_LEFT && !sel) *speed = (*speed - 1 < SPEED_SLOWEST) ? SPEED_SLOWEST : *speed - 1;
		else if(key & KEY_RIGHT && !sel) *speed = (*speed + 1 > SPEED_FASTEST) ? SPEED_FASTEST : *speed + 1;
		else if(key & (KEY_UP | KEY_DOWN)) sel = !sel;
	}
	
	closeWindow(110, 30);
	
	if(sel == 1 && key & KEY_ENTER) drawHiscores(0);
}


/************************************************
*
*  about() - Displays the about box
*
************************************************/

void about(void) {
	openWindow(110, 70);
	
	drawText(60, 18, "Snake", TEXT_OR | TEXT_BLACK);
	drawChar(38, 34, 44, TEXT_OR | TEXT_BLACK);
	drawText(50, 34, "Copyright", TEXT_OR | TEXT_DARK);
	drawText(64, 45, "2002", TEXT_OR | TEXT_DARK);
	drawText(36, 61, "Christopher", TEXT_OR | TEXT_BLACK);
	drawText(44, 72, "Gonterman", TEXT_OR | TEXT_BLACK);
	
	delay(50);
	while(!getKey());
	
	closeWindow(110, 70);
}


/************************************************
*
*  drawText() - Draws text using the custom font
*
*  int x       -      The x coord of the first character
*  int y       -      The y coord of the first character
*  const char *text - The text to display
*  int flags     -    How to draw the text:
*                      TEXT_GRAYINTERIOR
*                      TEXT_INTMASKNORMAL
*                      TEXT_INTWHITE
*                      TEXT_INTLIGHT
*                      TEXT_INTDARK
*                      TEXT_ASSUREOUTLINE
*                      TEXT_XOR
*                      TEXT_MASK
*                      TEXT_OR
*                      TEXT_WHITE
*                      TEXT_LIGHT
*                      TEXT_DARK
*                      TEXT_BLACK
*                      TEXT_NORMAL
*
************************************************/

void drawText(int x, int y, const char *text, int flags) {
	int i = 0, xOff = 0;//, sOff = 0;
	
	while(*(text+i)) {
		//sOff = pickChar(*(text + i));
		//if(sOff < 10)
			//DrawNum(x + xOff, y, sOff);
		//else// if(sOff < 42)
			drawChar(x + xOff, y, pickChar(*(text + i)), flags);
		//else
		//	EraseFont(x + xOff, y);
		xOff += 8;
		i++;
	}
}


/************************************************
*
*  drawChar() - Draws a character using the custom font
*
*  int x   -   The x coord of the character
*  int y   -   The y coord of the character
*  char c   -  The character to draw !not ASCII-encoded!
*  int flags - How to draw the character
*               See drawText() for details
*
************************************************/

void drawChar(int x, int y, char c, int flags) {
	if(c > 44)
		return;
	
	if(flags & TEXT_GRAYINTERIOR) {
		Sprite8_AND(x, y, 8, FontMask+c*8, GetPlane(DARK_PLANE));
		Sprite8_AND(x, y, 8, FontMask+c*8, GetPlane(LIGHT_PLANE));
	}
	else if(flags & TEXT_INTMASKNORMAL) {
		Sprite8_AND(x, y, 8, FontMask+c*8, LCD_MEM);
	}
	if(flags & TEXT_INTLIGHT) {
		InvertedSprite8(x, y, 8, FontMask+c*8, GetPlane(LIGHT_PLANE), ISPRITE_OR);
	}
	if(flags & TEXT_INTDARK) {
		InvertedSprite8(x, y, 8, FontMask+c*8, GetPlane(DARK_PLANE), ISPRITE_OR);
	}
	
	if(flags & TEXT_ASSUREOUTLINE) {
		Sprite8_OR(x, y, 8, Font+c*8, GetPlane(LIGHT_PLANE));
		Sprite8_OR(x, y, 8, Font+c*8, GetPlane(DARK_PLANE));
		Sprite8_XOR(x, y, 8, Font+c*8, GetPlane(LIGHT_PLANE));
		Sprite8_XOR(x, y, 8, Font+c*8, GetPlane(DARK_PLANE));
	}
	
	switch(flags & TEXT_METHOD) {
	case TEXT_XOR:
		if(flags & TEXT_LIGHT) Sprite8_XOR(x, y, 8, Font+c*8, GetPlane(LIGHT_PLANE));
		if(flags & TEXT_DARK) Sprite8_XOR(x, y, 8, Font+c*8, GetPlane(DARK_PLANE));
		if(flags & TEXT_NORMAL) Sprite8_XOR(x, y, 8, Font+c*8, LCD_MEM);
		break;
	case TEXT_MASK:
		if(flags & TEXT_LIGHT) Sprite8_AND(x, y, 8, FontMask+c*8, GetPlane(DARK_PLANE));
		if(flags & TEXT_DARK) Sprite8_AND(x, y, 8, FontMask+c*8, GetPlane(LIGHT_PLANE));
		if(flags & TEXT_NORMAL) Sprite8_AND(x, y, 8, FontMask+c*8, LCD_MEM);
		//break;
	case TEXT_OR:
		if(flags & TEXT_LIGHT) Sprite8_OR(x, y, 8, Font+c*8, GetPlane(LIGHT_PLANE));
		if(flags & TEXT_DARK) Sprite8_OR(x, y, 8, Font+c*8, GetPlane(DARK_PLANE));
		if(flags & TEXT_NORMAL) Sprite8_OR(x, y, 8, Font+c*8, LCD_MEM);
		break;
	}
}

/************************************************
*
*  pickChar() - Finds the encoding for a desired character
*
*  char c    -   The desired character !ASCII-encoded!
*
*  return char - The encoded character
*
************************************************/

char pickChar(char c) {
	if(c >= '0' && c <= '9') //a number
		return (c - '0');
	
	if(c >= 'a' && c <= 'z') //lowercase
		return (c - 'a' + 10);
	
	if(c >= 'A' && c <= 'Z') //uppercase
		return (c - 'A' + 10);
	
	switch(c) {
	case '-':
		return 36;
		break;
	case '+':
		return 37;
		break;
	case '=':
		return 38;
		break;
	case '_':
		return 39;
		break;
	case '.':
		return 40;
		break;
	case '!':
		return 41;
		break;
	case '?':
		return 42;
		break;
	case '#':
		return 43;
		break;
	default:
		return 127;
	}
}


/************************************************
*
*  showPass() - Displays the password for a level
*
*  int level - The level to display the password for
*
************************************************/

void showPass(int level) {
	int i;
	//                  0  0  0  0    4  Q   _   I     =   T   H   7    6  F   +   W     P   M   5   ?    .   J   2  Y     5  #   R   N     B   U   !   4    K   I   V   1
	char pass[9][4] = {{0, 0, 0, 0}, {4, 26, 39, 18}, {38, 29, 17, 7}, {6, 15, 37, 32}, {25, 22, 5, 42}, {40, 19, 2, 34}, {5, 43, 27, 23}, {11, 30, 41, 4}, {20, 18, 31, 1}};
	
	if(level > 9 || level < 1) return;
	
	ClearGrayScreen();
	
	drawText(48, 40, "Password", TEXT_OR | TEXT_LIGHT);
	for(i = 0; i < 4; i++) drawChar(64+i*8, 52, pass[level][i], TEXT_OR | TEXT_LIGHT);
	
	delay(100);
	
	ClearGrayScreen();
	
	drawText(48, 40, "Password", TEXT_OR | TEXT_DARK);
	for(i = 0; i < 4; i++) drawChar(64+i*8, 52, pass[level][i], TEXT_OR | TEXT_DARK);
	
	delay(100);
	
	//drawText(48, 40, "Password", TEXT_OR | TEXT_LIGHT);
	for(i = 0; i < 4; i++) drawChar(64+i*8, 52, pass[level][i], TEXT_OR | TEXT_LIGHT);
	
	while(!getKey());
	
	ClearGrayScreen();
	
}


/************************************************
*
*  askWarp() - Asks the player for level & password
*
*  return int - The selected level if the password was valid
*                0 if the password was invalid
*
************************************************/

int askWarp(void) {
	int i = 0, level = 0;
	int key = KEY_NONE, sel = 0, subSel = 0;
	char code[4] = {0, 0, 0, 0};
	//                  0  0  0  0    4  Q   _   I     =   T   H   7    6  F   +   W     P   M   5   ?    .   J   2  Y     5  #   R   N     B   U   !   4    K   I   V   1
	char pass[9][4] = {{0, 0, 0, 0}, {4, 26, 39, 18}, {38, 29, 17, 7}, {6, 15, 37, 32}, {25, 22, 5, 42}, {40, 19, 2, 34}, {5, 43, 27, 23}, {11, 30, 41, 4}, {20, 18, 31, 1}};
	//char pass[9][4] = {{0, 0, 0, 0}, {0, 0, 0, 0}, {0, 0, 0, 0}, {0, 0, 0, 0}, {0, 0, 0, 0}, {0, 0, 0, 0}, {0, 0, 0, 0}, {0, 0, 0, 0}, {0, 0, 0, 0}};
	char masterCode[4] = {9, 5, 4, 2};
	
	openWindow(110, 40);
	
	drawText(50, 38, "Level", TEXT_BLACK | TEXT_OR);
	drawText(40, 54, "Code", TEXT_BLACK | TEXT_OR);
	
	while(!(key & (KEY_ENTER | KEY_ESC))) {
		
		//DrawGrayRect(89, 43, 126, 53, COLOR_WHITE, RECT_FILLED);
		EraseFont(102, 38);
		DrawNum(102, 38, level);
		
		for(i = 0; i < 4; i++) {
			EraseFont(88 + i*8, 54);
			drawChar(88 + i*8, 54, code[i], TEXT_OR | TEXT_BLACK);
		}
		
		key = KEY_NONE;
		
		delay(50);
		while(!(key & (KEY_ENTER | KEY_ESC | KEY_LEFT | KEY_RIGHT | KEY_DOWN | KEY_UP))) {
			key = getKey();
			if(sel == 0) {
				if(subSel == 0) {
					XorArrow(43, 40, DIR_EAST);
					XorArrow(90, 40, DIR_WEST);
				}
				else {
					XorArrow(103, 32, DIR_SOUTH);
					XorArrow(103, 47, DIR_NORTH);
				}
			}
			else {
				if(subSel == 0) {
					XorArrow(34, 56, DIR_EAST);
					XorArrow(73, 56, DIR_WEST);
				}
				else {
					XorArrow(81 + subSel*8, 48, DIR_SOUTH);
					XorArrow(81 + subSel*8, 63, DIR_NORTH);
				}
			}
			optDelay(15);
		}
		
		if(sel == 0) {
			if(subSel == 0) {
				EraseArrow(43, 40);
				EraseArrow(90, 40);
			}
			else {
				EraseArrow(103, 32);
				EraseArrow(103, 47);
			}
		}
		else {
			if(subSel == 0) {
				EraseArrow(34, 56);
				EraseArrow(73, 56);
			}
			else {
				EraseArrow(81 + subSel*8, 48);
				EraseArrow(81 + subSel*8, 63);
			}
		}
		
		if(key & KEY_RIGHT) {
			if(sel == 0) subSel = !subSel;
			else subSel = (subSel >= 4) ? 0 : subSel + 1;
		}
		else if(key & KEY_LEFT) {
			if(sel == 0) subSel = !subSel;
			else subSel = (subSel <= 0) ? 4 : subSel - 1;
		}
		else if(key & KEY_DOWN) {
			if(subSel == 0)
				sel = !sel;
			else {
				if(sel == 0) level = (level - 1 < 0) ? 9 : level - 1;
				else {
					code[subSel-1] -= (key & KEY_2ND) ? 5 : 1;
					if(code[subSel-1] < 0) code[subSel-1] = code[subSel-1] += 43;
				}
			}
		}
		else if(key & KEY_UP) {
			if(subSel == 0)
				sel = !sel;
			else {
				if(sel == 0) level = (level + 1 > 9) ? 0 : level + 1;
				else {
					code[subSel-1] += (key & KEY_2ND) ? 5 : 1;
					if(code[subSel-1] > 43) code[subSel-1] -= 43;
				}
			}
		}
	}
	
	closeWindow(110, 40);
	
	if(key & KEY_ESC) return NO_LEVEL;
	
	for(i = 0; i < 4; i++)
		if(code[i] != pass[level][i] && code[i] != masterCode[i]) return LEVEL_0;
	
	return level;
}


/************************************************
*
*  checkHiscores() - Checks for a new high score
*
*  SCORE newScore - The score to check
*  int type    -    The type of game played
*
*  return int   -   1 if a new score was entered
*                   0 otherwise
*
************************************************/

int checkHiscores(SCORE newScore, int type) {
	SCORE scores[15];
	int i, place = 6, offset;
	
	if(!strcmp(newScore.level, "NONE")) return 0;
	
	loadScores(scores);
	
	switch(type) {
	case GAME_LEVELS:
	case GAME_WARP:
		offset = 0;
		break;
	case GAME_FREEPLAY:
		offset = 5;
		break;
	case GAME_RANDOM:
	case GAME_CUSTOM:
	default:
		offset = 10;
		break;	
	}
	
	place += offset;
	
	for(i = 4+offset; i >= offset; i--)
		if(scores[i].score < newScore.score) place = i;
	
	if(place >= offset+5) return 0;
	
	for(i = 4+offset; i > place; i--) {
		scores[i].score = scores[i-1].score;
		//scores[i].level = scores[i-1].level;
		sprintf(scores[i].level, "%s", scores[i-1].level);
	}
	scores[place].score = newScore.score;
	//scores[place].level = newScore.level;
	sprintf(scores[place].level, "%s", newScore.level);
	
	saveScores(scores);
	
	return 1;
}


/************************************************
*
*  drawHiscores() - Displays the hiscores
*
*  int sel - Which scoreboard to display first
*
************************************************/

void drawHiscores(int sel) {
	SCORE scores[15];
	int i, key = KEY_NONE, offset;
	char str[10];
	
	openWindow(120, 80);
	
	loadScores(scores);
	
	while(!(key & (KEY_ESC | KEY_ENTER))) {
		DrawGrayRect(20, 11, 140, 87, COLOR_WHITE, RECT_FILLED);
		
		if(sel == 0) {
			sprintf(str, "Levels");
			offset = 24;
		}
		else if(sel == 1) {
			sprintf(str, "Freeplay");
			offset = 32;
		}
		else {
			sprintf(str, "Custom");
			offset = 24;
		}
		drawText(80-offset, 14, str, TEXT_OR | TEXT_BLACK);
		
		XorArrow(71-offset, 15, DIR_WEST);
		XorArrow(84+offset, 15, DIR_EAST);
		
		drawText(26, 24, "Pl", TEXT_OR | TEXT_DARK);
		drawText(94, 24, "Score", TEXT_OR | TEXT_DARK);
		if(sel != 1)
			drawText(54, 24, "Lvl", TEXT_OR | TEXT_DARK);
		
		for(i = 0; i < 5; i++) {
			drawChar(30, 36+i*10, i+1, TEXT_OR | TEXT_BLACK);
			drawChar(38, 36+i*10, 40, TEXT_OR | TEXT_DARK);
			
			if(sel != 1) {
			//if(scores[i].level == LEVEL_FREEPLAY) sprintf(str, "F");
			//	if(scores[i+sel*5].level == CUSTOM_LEVEL) sprintf(str, "C");
			//	else if(scores[i+sel*5].level == RANDOM_LEVEL) sprintf(str, "R");
			//	else sprintf(str, "%d", scores[i+sel*5].level);
			//	drawText(62, 36+i*10, str, TEXT_OR | TEXT_BLACK);
				drawText(60, 36+i*10, scores[i+sel*5].level, TEXT_OR | TEXT_BLACK);
			}
			
			sprintf(str, "%6lu", scores[i+sel*5].score);
			drawText(86, 36+i*10, str, TEXT_OR | TEXT_BLACK);
		}
		
		delay(50);
		key = KEY_NONE;
		while(!(key = getKey()));
		
		if(key & KEY_LEFT) {
			if(--sel < 0) sel = 2;
		}
		else if(key & KEY_RIGHT) {
			if(++sel > 2) sel = 0;
		}
	}
	
	closeWindow(120, 80);
}


/************************************************
*
*  editor() - Edits levels
*
*  int args - How to use the editor:
*              EDITOR_NEW
*              EDITOR_LOAD
*
************************************************/

void editor(int args) {
	char *board = NULL;
	int key = KEY_NONE, result;
	POS mouse, upLeft;
	
	if(args == EDITOR_NEW)
		board = newEditBoard();
	else {
		board = loadBoard(NULL);
	}
	if(!board) return;
	
	mouse.x = BOARDWIDTH / 2;
	mouse.y = BOARDHEIGHT / 2;
	upLeft.x = 80 - (BOARDWIDTH*5/2);
	upLeft.y = 50 - (BOARDHEIGHT*5/2);
	
	drawBoard(board, 0);
	DrawSnakeHead(MIDBOARD, BOTTOMBOARD+8, DIR_NORTH);
	
	key = KEY_NONE;
	while(!(key & (KEY_ENTER | KEY_ESC))) {
		XorMouse(upLeft.x + mouse.x * 5, upLeft.y + mouse.y * 5);
		key = KEY_NONE;
		delay(50);
		while(!(key = getKey()));
		if(key & KEY_BOSS) {
			free(board);
			forceExit();
		}
		XorMouse(upLeft.x + mouse.x * 5, upLeft.y + mouse.y * 5);
		if(key & KEY_LEFT) {
			if(--mouse.x < 0) mouse.x = BOARDWIDTH-1;
		}
		else if(key & KEY_RIGHT) {
			if(++mouse.x == BOARDWIDTH) mouse.x = 0;
		}
		if(key & KEY_DOWN) {
			if(++mouse.y == BOARDHEIGHT) mouse.y = 0;
		}
		else if(key & KEY_UP) {
			if(--mouse.y < 0) mouse.y = BOARDHEIGHT-1;
		}
		if(key & KEY_2ND && !(mouse.x == BOARDWIDTH/2-1 && mouse.y == BOARDHEIGHT-1)) {
			BOARDPOS(mouse.x, mouse.y) = BOARD_OBSTACLE;
			int temp = getObstType(board, mouse.x, mouse.y);
			if(BOARDWRAP) {
				if(temp & OBST_SOUTH) {
					if(mouse.y < BOARDHEIGHT-1)
						DrawMultiObst(upLeft.x + mouse.x*5, upLeft.y + (mouse.y+1)*5, getObstType(board, mouse.x, mouse.y+1));
					else
						DrawMultiObst(upLeft.x + mouse.x*5, upLeft.y, getObstType(board, mouse.x, 0));
				}
				if(temp & OBST_WEST) {
					if(mouse.x > 0)
						DrawMultiObst(upLeft.x + (mouse.x-1)*5, upLeft.y + mouse.y*5, getObstType(board, mouse.x-1, mouse.y));
					else
						DrawMultiObst(upLeft.x + (BOARDWIDTH-1)*5, upLeft.y + mouse.y*5, getObstType(board, BOARDWIDTH-1, mouse.y));
				}
				if(temp & OBST_NORTH) {
					if(mouse.y > 0)
						DrawMultiObst(upLeft.x + mouse.x*5, upLeft.y + (mouse.y-1)*5, getObstType(board, mouse.x, mouse.y-1));
					else
						DrawMultiObst(upLeft.x + mouse.x*5, upLeft.y + (BOARDHEIGHT-1)*5, getObstType(board, mouse.x, BOARDHEIGHT-1));
				}
				if(temp & OBST_EAST) {
					if(mouse.x < BOARDWIDTH-1)
						DrawMultiObst(upLeft.x + (mouse.x+1)*5, upLeft.y + mouse.y*5, getObstType(board, mouse.x+1, mouse.y));
					else
						DrawMultiObst(upLeft.x, upLeft.y + mouse.y*5, getObstType(board, 0, mouse.y));
				}
			}
			else {
				if(temp & OBST_SOUTH) DrawMultiObst(upLeft.x + mouse.x*5, upLeft.y + (mouse.y+1)*5, getObstType(board, mouse.x, mouse.y+1));
				if(temp & OBST_WEST) DrawMultiObst(upLeft.x + (mouse.x-1)*5, upLeft.y + mouse.y*5, getObstType(board, mouse.x-1, mouse.y));
				if(temp & OBST_NORTH) DrawMultiObst(upLeft.x + mouse.x*5, upLeft.y + (mouse.y-1)*5, getObstType(board, mouse.x, mouse.y-1));
				if(temp & OBST_EAST) DrawMultiObst(upLeft.x + (mouse.x+1)*5, upLeft.y + mouse.y*5, getObstType(board, mouse.x+1, mouse.y));
			}
			DrawMultiObst(upLeft.x + mouse.x*5, upLeft.y + mouse.y*5, temp);
		}
		else if(key & KEY_DMND) {
			BOARDPOS(mouse.x, mouse.y) = BOARD_EMPTY;
			int temp = getObstType(board, mouse.x, mouse.y);
			if(BOARDWRAP) {
				if(temp & OBST_SOUTH) {
					if(mouse.y < BOARDHEIGHT-1)
						DrawMultiObst(upLeft.x + mouse.x*5, upLeft.y + (mouse.y+1)*5, getObstType(board, mouse.x, mouse.y+1));
					else
						DrawMultiObst(upLeft.x + mouse.x*5, upLeft.y, getObstType(board, mouse.x, 0));
				}
				if(temp & OBST_WEST) {
					if(mouse.x > 0)
						DrawMultiObst(upLeft.x + (mouse.x-1)*5, upLeft.y + mouse.y*5, getObstType(board, mouse.x-1, mouse.y));
					else
						DrawMultiObst(upLeft.x + (BOARDWIDTH-1)*5, upLeft.y + mouse.y*5, getObstType(board, BOARDWIDTH-1, mouse.y));
				}
				if(temp & OBST_NORTH) {
					if(mouse.y > 0)
						DrawMultiObst(upLeft.x + mouse.x*5, upLeft.y + (mouse.y-1)*5, getObstType(board, mouse.x, mouse.y-1));
					else
						DrawMultiObst(upLeft.x + mouse.x*5, upLeft.y + (BOARDHEIGHT-1)*5, getObstType(board, mouse.x, BOARDHEIGHT-1));
				}
				if(temp & OBST_EAST) {
					if(mouse.x < BOARDWIDTH-1)
						DrawMultiObst(upLeft.x + (mouse.x+1)*5, upLeft.y + mouse.y*5, getObstType(board, mouse.x+1, mouse.y));
					else
						DrawMultiObst(upLeft.x, upLeft.y + mouse.y*5, getObstType(board, 0, mouse.y));
				}
			}
			else {
				if(temp & OBST_SOUTH) DrawMultiObst(upLeft.x + mouse.x*5, upLeft.y + (mouse.y+1)*5, getObstType(board, mouse.x, mouse.y+1));
				if(temp & OBST_WEST) DrawMultiObst(upLeft.x + (mouse.x-1)*5, upLeft.y + mouse.y*5, getObstType(board, mouse.x-1, mouse.y));
				if(temp & OBST_NORTH) DrawMultiObst(upLeft.x + mouse.x*5, upLeft.y + (mouse.y-1)*5, getObstType(board, mouse.x, mouse.y-1));
				if(temp & OBST_EAST) DrawMultiObst(upLeft.x + (mouse.x+1)*5, upLeft.y + mouse.y*5, getObstType(board, mouse.x+1, mouse.y));
			}
			EraseSnake(upLeft.x + mouse.x * 5, upLeft.y + mouse.y * 5);
		}
		if(key & KEY_F5) {
			BOARDWRAP = !BOARDWRAP;
			drawBoard(board, 0);
			DrawSnakeHead(MIDBOARD, BOTTOMBOARD+8, DIR_NORTH);
		}
	}
	
	if(key & KEY_ESC) {
		free(board);
		return;
	}
	
	while((result = saveBoard(board)) <= 0) {
		if(result == SAVE_FAILED) {
			drawText(24, 46, "Unable to save", TEXT_OR | TEXT_BLACK);
			delay(50);
			while(!getKey());
		}
		else if(result == SAVE_INVALIDNAME) {
			drawText(32, 46, "Invalid name", TEXT_OR | TEXT_BLACK);
			delay(50);
			while(!getKey());
		}
	}
	
	free(board);
}


/************************************************
*
*  newEditBoard() - Creates a new board for editing
*
*  return char* - The new board, NULL on abort or failure
*
************************************************/

char *newEditBoard(void) {
	char *board = NULL;
	char dim[2] = {DEFAULT_WIDTH, DEFAULT_HEIGHT};
	int sel = 0, key = KEY_NONE;
	
	openWindow(110, 30);
	
	drawText(35, 41, "Width", TEXT_OR | TEXT_BLACK);
	drawText(35, 51, "Height", TEXT_OR | TEXT_BLACK);
	
	while(!(key & (KEY_ENTER | KEY_ESC))) {
		DrawGrayRect(98, 41, 129, 59, COLOR_WHITE, RECT_FILLED);
		DrawNum(105, 41, dim[0]/10);
		DrawNum(115, 41, dim[0]%10);
		DrawNum(105, 51, dim[1]/10);
		DrawNum(115, 51, dim[1]%10);
		key = KEY_NONE;
		delay(50);
		while(!(key & (KEY_ENTER | KEY_ESC | KEY_LEFT | KEY_RIGHT | KEY_DOWN | KEY_UP))) {
			key = getKey();
			XorArrow(99, 42+sel*10, DIR_EAST);
			XorArrow(124, 42+sel*10, DIR_WEST);
			optDelay(15);
		}
		if(key & KEY_LEFT) {
			if(--dim[sel] < 10) dim[sel] = (sel ? DEFAULT_HEIGHT : DEFAULT_WIDTH);
		}
		else if(key & KEY_RIGHT) {
			if(++dim[sel] > (sel ? DEFAULT_HEIGHT : DEFAULT_WIDTH)) dim[sel] = 10;
		}
		else if(key & (KEY_UP | KEY_DOWN)) {
			sel = !sel;
		}
	}
	
	closeWindow(110, 30);
	
	if(key & KEY_ESC) return NULL;
	
	board = (char *)malloc(dim[0] * dim[1] + BOARD_TO_DATA);
	if(!board) return NULL;
	
	memset(board, 0, dim[0] * dim[1] + BOARD_TO_DATA);
	
	BOARDWIDTH = dim[0];
	BOARDHEIGHT = dim[1];
	BOARDMINAPPLES = 1;
	BOARDMAXAPPLES = 10;
	
	return board;
}


/************************************************
*
*  saveBoard() - Prompts for level attributes, 
*                 then saves a level
*
*  char *board - The board to save
*
*  return int  - Success/failure report:
*                  SAVE_ESCKEY
*                  SAVE_INVALIDNAME
*                  SAVE_FAILED
*                  SAVE_SUCCESS
*
************************************************/

int saveBoard(char *board) {
	char str[10] = "";
	int i, sel = 0, subSel = 0, charLoc = 0, key = KEY_NONE;
	int apples[2] = {BOARDMINAPPLES, BOARDMAXAPPLES};
	int wrap = BOARDWRAP;
	
	openWindow(120, 64);
	
	drawText(27, 23, "Name", TEXT_OR | TEXT_BLACK);
	drawText(27, 37, "Apple Min", TEXT_OR | TEXT_BLACK);
	drawText(27, 51, "Apple Max", TEXT_OR | TEXT_BLACK);
	drawText(27, 65, "Side Wrap", TEXT_OR | TEXT_BLACK);
	
	key = KEY_NONE;
	sel = 0;
	subSel = 0;
	while(key != OSKEY_ENTER && key != OSKEY_ESC) {
		DrawGrayRect(65, 23, 134, 32, COLOR_WHITE, RECT_FILLED);
		drawText(65, 23, str, TEXT_OR | TEXT_BLACK);
		DrawGrayRect(114, 37, 128, 59, COLOR_WHITE, RECT_FILLED);
		DrawNum(114, 37, apples[0]/10);
		DrawNum(122, 37, apples[0]%10);
		DrawNum(114, 51, apples[1]/10);
		DrawNum(122, 51, apples[1]%10);
		DrawGrayRect(110, 65, 133, 73, COLOR_WHITE, RECT_FILLED);
		if(wrap)
			drawText(110, 65, "Yes", TEXT_OR | TEXT_BLACK);
		else
			drawText(114, 65, "No", TEXT_OR | TEXT_DARK);
		key = KEY_NONE;
		delay(50);
		while(!key) {
			key = getAlpha();
			if(sel == 0) {
				if(subSel == 0) {
					XorArrow(21, 24, DIR_EAST);
					XorArrow(59, 24, DIR_WEST);
				}
				else {
					FastDrawLine(GetPlane(DARK_PLANE), 66+charLoc*8, 22, 65+charLoc*8, 30, A_XOR);
					FastDrawLine(GetPlane(LIGHT_PLANE), 66+charLoc*8, 26, 66+charLoc*8, 30, A_XOR);
				}
			}
			else {
				if(subSel == 0) {
					XorArrow(21, 24+sel*14, DIR_EAST);
					XorArrow(99, 24+sel*14, DIR_WEST);
				}
				else {
					XorArrow(120, 17+sel*14, DIR_NORTH);
					XorArrow(120, 32+sel*14, DIR_SOUTH);
				}
			}
			optDelay(15);
		}
		
		if(sel == 0) {
			if(subSel == 0) {
				EraseArrow(21, 24);
				EraseArrow(59, 24);
			}
			else {
				FastDrawLine(GetPlane(DARK_PLANE), 66+charLoc*8, 22, 65+charLoc*8, 30, A_REVERSE);
				FastDrawLine(GetPlane(LIGHT_PLANE), 66+charLoc*8, 26, 66+charLoc*8, 30, A_REVERSE);
			}
		}
		else {
			if(subSel == 0) {
				EraseArrow(21, 24+sel*14);
				EraseArrow(99, 24+sel*14);
			}
			else {
				EraseArrow(120, 17+sel*14);
				EraseArrow(120, 32+sel*14);
			}
		}
		
		if(key == OSKEY_LEFT)
			subSel = !subSel;
		else if(key == OSKEY_RIGHT)
			subSel = !subSel;
		else if(key == OSKEY_UP) {
			if(!subSel) {
				if(--sel < 0) sel = 3;
			}
			else if(sel > 0 && sel < 3) {
				if(++apples[sel-1] > 50) apples[sel-1] = 1;
			}
			else if(sel == 3)
				wrap = !wrap;
		}
		else if(key == OSKEY_DOWN) {
			if(!subSel) {
				if(++sel > 3) sel = 0;
			}
			else if(sel > 0 && sel < 3) {
				if(--apples[sel-1] < 1) apples[sel-1] = 50;
			}
			else if (sel == 3)
				wrap = !wrap;
		}
		
		if(sel == 0 && subSel) {
			if((key >= 'A' && key <= 'Z') || key == OSKEY_SPACE) {
				str[charLoc] = key;
				if(charLoc < 7) charLoc++;
			}
			else if(key == OSKEY_BACKSPACE) {
				if(charLoc > 0)
					charLoc--;
				str[charLoc] = 0;
			}
		}
	}
	
	if(key == OSKEY_ESC) return SAVE_ESCKEY;
	
	for(i = 7; i >= 0; i--)
		if(str[i] == ' ')
			str[i] = '_';
	
	wipeScreen();
	
	if(str[0] == 0) return SAVE_INVALIDNAME;
	
	BOARDMINAPPLES = apples[0];
	BOARDMAXAPPLES = apples[1];
	BOARDWRAP = wrap;
	
	return saveBoardFile(board, str);
}


/************************************************
*
*  loadBoard() - Loads a board from memory
*
*  char *name  -  To store level name, NULL if unnecessary
*
*  return char* - Points to the loaded board,
*                  NULL if unsuccessful
*
************************************************/

char* loadBoard(char *name) {
	char str[10] = "";
	int i, sel = 0, key = KEY_NONE;
	
	openWindow(120, 24);
	
	drawText(27, 46, "Name", TEXT_OR | TEXT_BLACK);
	
	key = KEY_NONE;
	sel = 0;
	while(key != OSKEY_ENTER && key != OSKEY_ESC) {
		DrawGrayRect(65, 45, 134, 55, COLOR_WHITE, RECT_FILLED);
		drawText(65, 46, str, TEXT_OR | TEXT_BLACK);
		key = KEY_NONE;
		delay(50);
		while(!key) {
			key = getAlpha();
			FastDrawLine(GetPlane(DARK_PLANE), 66+sel*8, 45, 65+sel*8, 53, A_XOR);
			FastDrawLine(GetPlane(LIGHT_PLANE), 66+sel*8, 49, 66+sel*8, 53, A_XOR);
			optDelay(15);
		}
		
		FastDrawLine(GetPlane(DARK_PLANE), 66+sel*8, 45, 65+sel*8, 53, A_REVERSE);
		FastDrawLine(GetPlane(LIGHT_PLANE), 66+sel*8, 49, 66+sel*8, 53, A_REVERSE);
		
		if((key >= 'A' && key <= 'Z') || key == OSKEY_SPACE) {
			str[sel] = key;
			if(sel < 8) sel++;
		}
		else if(key == OSKEY_BACKSPACE) {
			if(sel > 0)
				sel--;
			str[sel] = 0;
		}
	}
	
	closeWindow(120, 24);
	
	for(i = 7; i >= 0; i--)
		if(str[i] == ' ')
			str[i] = '_';
	
	if(name && key != OSKEY_ESC) sprintf(name, str);
	
	return loadBoardFile(str);
}


/************************************************
*
*  askRandomDiff() - Prompts for difficulty
*
*  return int - The difficulty chosen
*
************************************************/

int askRandomDiff() {
	int key = KEY_NONE;//, i;
	static int diff = 0;
	
	openWindow(120, 24);
	
	drawText(27, 46, "Difficulty", TEXT_OR | TEXT_BLACK);
	
	key = KEY_NONE;
	while(!(key & (KEY_ENTER | KEY_ESC))) {
		EraseFont(127, 46);
		DrawNum(127, 46, diff);
		key = KEY_NONE;
		delay(50);
		while(!(key & (KEY_ENTER | KEY_ESC | KEY_DOWN | KEY_UP))) {
			key = getKey();
			XorArrow(128, 40, DIR_NORTH);
			XorArrow(128, 55, DIR_SOUTH);
			optDelay(15);
		}
		
		EraseArrow(128, 40);
		EraseArrow(128, 55);
		
		if(key & KEY_UP) {
			if(++diff > 9) diff = 0;
		}
		else if(key & KEY_DOWN) {
			if(--diff < 0) diff = 9;
		}
	}
	
	closeWindow(120, 24);
	
	//if(key & KEY_ESC) return NULL;
	if(key & KEY_ESC) return -1;
	
	//return makeRandomBoard(diff);
	return diff;
}


/************************************************
*
*  makeRandomBoard() - Contructs a random board
*
*  int *_diff - How difficult to make the board, -1 to prompt
*
*  return char* - The random board, NULL if unsuccessful
*
************************************************/

char* makeRandomBoard(int *_diff) {
	if(*_diff < 0) *_diff = askRandomDiff();
	if(*_diff < 0) return NULL;
	
	int diff = *_diff;
	char *board = NULL;
	int i = 0, j = 0;
	POS pos;
	int width = DEFAULT_WIDTH-diff/2, height = DEFAULT_HEIGHT-diff/3;
	
	board = malloc(width * height + BOARD_TO_DATA);
	memset(board, 0, width * height + BOARD_TO_DATA);
	
	BOARDWIDTH = width;
	BOARDHEIGHT = height;
	BOARDMINAPPLES = diff ? 1 : 2;
	BOARDMAXAPPLES = diff*2 + 10;
	
	//randomize();
	//if(pos.x != width/2-1 || pos.y+j < height-4)
	
	for(i = 0; i < (diff+5)*4; i++) {
		pos.x = rand() % (width / 2) + ((i & 1) ? (width/2)-1 : 1);
		pos.y = rand() % (height / 2) + ((i & 2) ? (height/2)-1 : 1);
		switch(rand() % 7) {
		case 0:
			for(j = -1; j < 2; j++)
				BOARDPOS(pos.x, pos.y+j) = BOARD_OBSTACLE;
			break;
		case 1:
			for(j = -1; j < 2; j++)
				BOARDPOS(pos.x+j, pos.y) = BOARD_OBSTACLE;
			break;
		case 2:
			for(j = 0; j < 2; j++)
				BOARDPOS(pos.x, pos.y+j) = BOARD_OBSTACLE;
			break;
		case 3:
			for(j = 0; j < 2; j++)
				BOARDPOS(pos.x+j, pos.y) = BOARD_OBSTACLE;
			break;
		default:
			BOARDPOS(pos.x, pos.y) = BOARD_OBSTACLE;
			break;
		}
	}
	
	for(i = 0; i < 4; i++) {
		BOARDPOS(width/2-1, height-i) = BOARD_EMPTY;
	}
	
	return board;
}


/************************************************
*
*  InvertedSprite8() - Inverts a 8xh sprite (on <-> off),
*                       then draws it
*
*  int x         -         The x coord to draw to
*  int y         -         The y coord to draw to
*  int h         -         The height of the sprite
*  unsigned char *sprite - The sprite to draw
*  void *dest       -      Where to draw to
*  int method       -      How to draw the sprite:
*                           ISprite_XOR
*                           ISprite_OR
*                           ISprite_AND
*
************************************************/

void InvertedSprite8(int x, int y, int h, unsigned char *sprite, void *dest, int method) {
	unsigned char *iSprite = NULL;
	int i;
	
	iSprite = (unsigned char*)malloc(h);
	for(i = 0; i < h; i++) iSprite[i] = ~sprite[i];
	
	if(method == ISPRITE_OR) Sprite8_OR(x, y, h, iSprite, dest);
	if(method == ISPRITE_AND) Sprite8_AND(x, y, h, iSprite, dest);
	if(method == ISPRITE_XOR) Sprite8_XOR(x, y, h, iSprite, dest);
	
	free(iSprite);
}


/************************************************
*
*  openWindow() - Opens a new graphical "window"
*
*  int width      -      The width of the window
*  int height      -     The height of the window (multiple of 2)
*
************************************************/

void openWindow(int width, int height) {
	height >>= 1;
	width >>= 1;
	
	int startX = 80 - width, endX = 80 + width, i = 0;
	
	for(; i < height; i++) {
		FastDrawGrayHLine(startX-1, endX+1, 49-i, COLOR_BLACK);
		FastDrawGrayHLine(startX-1, endX+1, 49+i, COLOR_DARKGRAY);
		FastDrawGrayHLine(startX, endX, 50-i, COLOR_WHITE);
		FastDrawGrayHLine(startX, endX, 48+i, COLOR_WHITE);
		delay(5);
	}
}


/************************************************
*
*  closeWindow() - Closes a graphical "window"
*
*  int width      -      The width of the window
*  int height      -     The height of the window (multiple of 2)
*
************************************************/

void closeWindow(int width, int height) {
	height >>= 1;
	width >>= 1;
	
	int startX = 79 - width, endX = 81 + width, i = height;
	
	for(; i >= 0; i--) {
		FastDrawGrayHLine(startX, endX, 49-i, COLOR_WHITE);
		FastDrawGrayHLine(startX, endX, 49+i, COLOR_WHITE);
		delay(5);
	}
}


/************************************************
*
*  wipeScreen() - Clears the screen by draw lines
*                  and erasing them from top to bottom
*
************************************************/

void wipeScreen(void) {
	int i = 4;
	for(; i < 104; i++) {
		FastDrawGrayHLine(0, 160, i, COLOR_BLACK);
		FastDrawGrayHLine(0, 160, i-1, COLOR_DARKGRAY);
		FastDrawGrayHLine(0, 160, i-2, COLOR_LIGHTGRAY);
		FastDrawGrayHLine(0, 160, i-3, COLOR_LIGHTGRAY);
		FastDrawGrayHLine(0, 160, i-4, COLOR_WHITE);
		delay(2);
	}
}


/************************************************
*
*  getObstType() - Finds what type of obstacle to place
*
*  char *board - The board to check against
*  int x    -    The x coord of the obstacle
*  int y    -    The y coord of the obstacle
*
*  return int - The kind of obstacle to use
*
************************************************/

int getObstType(char *board, int x, int y) {
	int temp = 0;
	
	if(!BOARDWRAP) {
		if(x < 0) {
			if(BOARDPOS(x+1, y)) return 0b1011;
			else return 0b1010;
		}
		else if(y < 0) {
			if(BOARDPOS(x, y+1)) return 0b1101;
			else return 0b0101;
		}
		else if(x >= BOARDWIDTH) {
			if(BOARDPOS(x-1, y)) return 0b1110;
			else return 0b1010;
		}
		else if(y >= BOARDHEIGHT) {
			if(BOARDPOS(x, y-1)) return 0b0111;
			else return 0b0101;
		}
	}
	
	if(y < BOARDHEIGHT-1) {
		if(BOARDPOS(x, y+1)) temp |= OBST_SOUTH;
	}
	else {
		if(BOARDWRAP) {
			if(BOARDPOS(x, 0))
				temp |= OBST_SOUTH;
		}
		else
			temp |= OBST_SOUTH;
	}
	if(x > 0) {
		if(BOARDPOS(x-1, y)) temp |= OBST_WEST;
	}
	else {
		if(BOARDWRAP) {
			if(BOARDPOS(BOARDWIDTH-1, y))
				temp |= OBST_WEST;
		}
		else
			temp |= OBST_WEST;
	}
	if(y > 0) {
		if(BOARDPOS(x, y-1)) temp |= OBST_NORTH;
	}
	else {
		if(BOARDWRAP) {
			if(BOARDPOS(x, BOARDHEIGHT-1))
				temp |= OBST_NORTH;
		}
		else
			temp |= OBST_NORTH;
	}
	if(x < BOARDWIDTH-1) {
		if(BOARDPOS(x+1, y)) temp |= OBST_EAST;
	}
	else {
		if(BOARDWRAP) {
			if(BOARDPOS(0, y))
				temp |= OBST_EAST;
		}
		else
			temp |= OBST_EAST;
	}
	
	return temp;
}
